@@ -225,10 +225,6 @@ html_view_relayout (HtmlView *view)
 		gtk_idle_remove (view->relayout_idle_id);
 		view->relayout_idle_id = 0;
 	}
-	if (GTK_WIDGET_HAS_FOCUS (view) && 
-	    view->document->focus_element == NULL) {
-		html_view_focus (HTML_VIEW (view), GTK_DIR_TAB_FORWARD);
-	}
 }
 
 static gint
@@ -587,6 +583,42 @@ html_view_setup_document (HtmlView *view)
 		html_view_build_tree (view, dom_Node_mkref (xmlDocGetRootElement ((xmlDoc *)DOM_NODE (view->document->dom_document)->xmlnode)));
 }
 
+static void
+html_view_disconnect_document (HtmlView *view, HtmlDocument *document)
+{
+	g_signal_handlers_disconnect_matched (G_OBJECT (view->document),
+					      G_SIGNAL_MATCH_FUNC,
+					      0, 0, NULL,
+					      html_view_inserted, NULL);
+
+	g_signal_handlers_disconnect_matched (G_OBJECT (view->document),
+					      G_SIGNAL_MATCH_FUNC,
+					      0, 0, NULL,
+					      html_view_removed, NULL);
+
+	g_signal_handlers_disconnect_matched (G_OBJECT (view->document),
+					      G_SIGNAL_MATCH_FUNC,
+					      0, 0, NULL,
+					      html_view_text_updated, NULL);
+
+	g_signal_handlers_disconnect_matched (G_OBJECT (view->document),
+					      G_SIGNAL_MATCH_FUNC,
+					      0, 0, NULL,
+					      html_view_style_updated, NULL);
+
+	g_signal_handlers_disconnect_matched (G_OBJECT (view->document),
+					      G_SIGNAL_MATCH_FUNC,
+					      0, 0, NULL,
+					      html_view_relayout_callback, 
+					      NULL);
+
+	g_signal_handlers_disconnect_matched (G_OBJECT (view->document),
+					      G_SIGNAL_MATCH_FUNC,
+					      0, 0, NULL,
+					      html_view_repaint_callback, 
+					      NULL);
+}
+
 static void
 html_view_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -715,7 +747,6 @@ html_view_focus_in (GtkWidget *widget, GdkEventFocus *event)
 	element = html_view_get_saved_focus (view);
 	if (element) {
 		html_document_update_focus_element (view->document, element);
-		html_view_scroll_to_node (view, DOM_NODE (view->document->focus_element), HTML_VIEW_SCROLL_TO_BOTTOM);
 	}
 	return FALSE;
 }
@@ -896,10 +927,7 @@ html_view_set_document (HtmlView *view, HtmlDocument *document)
 		g_object_ref (document);
 	
 	if (view->document != NULL) {
-		g_signal_handlers_disconnect_matched (G_OBJECT (view->document),
-						      G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL,
-						      NULL, view);
+		html_view_disconnect_document (view, view->document);
 		
 		html_view_layout_tree_free (view, view->root);
 	}
